

#ifdef DEBUG

static void da_return (int rv)
{
}

#define DA_RETURN(rv)   do { da_return (rv); return (rv); } while (0)
#else
#define DA_RETURN(rv)   return (rv)
#endif


typedef struct dockapp_object {
    void (*destructor) (dockapp_objectp);
    size_t size;
} dockapp_object;


static void da_trivial_dtor (dockapp_objectp)
{
}

static dockapp_objectp *da_newobj (
        size_t size, void (*dtor) (dockapp_objectp))
{
    dockapp_objectp *obj = malloc (size);
    assert (obj);

    obj->destructor = dtor;
    obj->size = size;
}

#define DA_DECL_CAST(subclass,name,objptr) \
    subclass *name; \
    if (objptr && objptr->size == sizeof (subclass)) \
        name = (subclass *)(objptr);

#define DA_DECL_CAST_CHECKED(subclass,name,objptr) \
    DA_DECL_CAST(subclass,name,objptr) \
    else \
        DA_RETURN (dockapp_rv_bad_obj);



void dockapp_delete (dockapp_objectp obj)
{
    if (!obj)
        return;
    obj->destructor (obj);
    free (obj);
}

/**********************************************************************
 * STARTUP
 */

dockapp_rv_t dockapp_init (
        const char *argv,
        const char **xpmdata)
{
    Display *display = XOpenDisplay (0);
    int screen = DefaultScreen (display);
    Window root = RootWindow (display, screen);
    int xfd = XConnectionNumber (display);
    XpmIcon appicon;
    XSizeHints winszhints;
    Window main_win, icon_win;
    XClassHint classhint;
    XWMHints wmhints;


    /* construct the pixmap */
    appicon.attributes.valuemask = XpmReturnPixels | XpmReturnExtensions;

    if (XpmCreatePixmapFromData (
            display,
            root,
            xpmdata,
            &appicon.pixmap,
            &appicon.mask,
            &appicon.attributes) != XpmSuccess) {
        return dockapp_rv_x_error;
    }

    /* construct the window */
    winszhints.flags = USSize | USPosition;
    winszhints.x = winszhints.y = 0;
    winszhints.width = winszhints.height = 64;

    main_win = XCreateSimpleWindow (
            display,
            root,               /* parent */
            0, 0, 64, 64,       /* pos & size */
            1,                  /* border width */
            GetColor ("white"), /* foreground */
            GetColor ("black"));/* background */

    icon_win = XCreateSimpleWindow (
            display,
            main_win,           /* parent */
            0, 0, 64, 64,       /* pos & size */
            1,                  /* border width */
            GetColor ("white"), /* foreground */
            GetColor ("black"));/* background */

    /* construct the shaping bitmap */

    /* set wm hints */
    classhint.res_name = appname;
    classhint.res_class = appname;
    XSetClassHint (display, main_win, &classhint);

    XStoreName (display, main_win, appname);


    wmhints.initial_state = WithdrawnState;
    wmhints.icon_window = icon_win;
    wmhints.window_group = main_win;
    XSetWMHints (display, win, &wmhints);

    /* finally, show the window */
    XMapWindow (display, win);



    
}


/**********************************************************************
 * CLICKREGIONS
 */
struct da_clickregion {
    dockapp_object base;

    int x, y, w, h;
    int buttonmask;

    dockapp_callback_rv_t (*cb) (void *, int x, int y);
    void *cbdata;
};

#define DA_MAX_CLICKREGIONS 50

static da_clickregion *da_clickregions[DA_MAX_CLICKREGIONS]
static size_t da_num_clickregions = 0;

static dockapp_callback_rv_t da_scan_clickregions (
        int hitx, int hity, int buttons)
{
    da_clickregion **c, **end;

    end = da_clickregions + da_num_clickregions;

    for (c = da_clickregions; c < end; ++c) {
        if ((buttons & c->buttonmask == c->buttonmask)
                && hitx >= x && hitx < x + w
                && hity >= y && hity < y + h) {
            if (c->cb (c->cbdata, hitx, hity) == dockapp_exit)
                return dockapp_exit;
        }
    }

    return dockapp_ok;
}

void da_clickregion_dtor (dockapp_objectp obj)
{
    da_clickregion **scan;
    DA_DECL_CAST (da_clickregion, c, obj);

    end = da_clickregions + da_num_clickregions;

    for (scan = da_clickregions; scan < end; ++scan) {
        if (*scan == obj) {
            memmove (scan, scan + 1, end - scan - 1);
            --da_num_clickregions;
            return;
        }
    }

}

dockapp_objectp dockapp_create_clickregion (
        int x, int y, int w, int h,
        int buttonmask,
        dockapp_callback_rv_t (*cb) (void *, int x, int y),
        void *cbdata)
{
    da_clickregion *c;

    if (da_num_clickregions >= DA_MAX_CLICKREGIONS)
        return 0;

    c = da_newobj (sizeof (da_clickregion), da_clickregion_dtor);

    c->x = x;
    c->y = y;
    c->w = w;
    c->h = h;
    c->buttonmask = buttonmask;
    c->cb = cb;
    c->cbdata = cbdata;

    da_clickregions[da_num_clickregions++] = c;
}

/**********************************************************************
 * FONT/TEXT MANAGEMENT
 */

#define DA_ASCII_BEGIN  0
#define DA_ASCII_END    128

struct da_fixed_ascii_font {
    dockapp_object base;

    /* glyph dimensions */
    int w, int h;

    /* for each character we store the coords of its source, or (-1, -1)
     * for missing ones
     */
    int x[DA_ASCII_END - DA_ASCII_BEGIN];
    int y[DA_ASCII_END - DA_ASCII_BEGIN];
};


dockapp_objectp dockapp_create_fixed_ascii_font (
        int charw, int charh)
{
    int ch;
    da_fixed_ascii_font *f
        = da_newobj (sizeof (da_fixed_ascii_font), da_trivial_dtor);

    f->w = charw;
    f->h = charh;

    for (ch = 0; ch < DA_ASCII_END - DA_ASCII_BEGIN; ++ch)
        f->x[ch] = f->y[ch] = -1;

    return (dockapp_objectp)f;
}


int dockapp_add_font_glyphs (
        dockapp_objectp obj,
        int x, int y,
        int first_char, int last_char)
{
    int ch;
    DA_DECL_CAST_CHECKED (da_fixed_ascii_font, f, obj);

    if (first_char < DA_ASCII_BEGIN || first_char >= DA_ASCII_END
        || last_char < DA_ASCII_BEGIN || last_char >= DA_ASCII_END)
        return dockapp_rv_bad_char;

    for (ch = first_char; ch < last_char; ++ch) {
        f->x[ch - DA_ASCII_BEGIN] = x;
        x += f->w;
        f->y[ch - DA_ASCII_BEGIN] = y;
    }

    return dockapp_rv_ok;
}


int dockapp_draw_text (
        dockapp_objectp font,
        int x, int y, int w, int h,
        const char *text,
        int (*pixel_mover) (sx, sy, tx, ty, w, h))
{
    DA_DECL_CAST_CHECKED (da_fixed_ascii_font, f, obj);
    
    for ( ; text && *text && w > 0; ++text) {
        if (*text < DA_ASCII_BEGIN || *text >= DA_ASCII_END
            || *text < DA_ASCII_BEGIN || *text >= DA_ASCII_END)
            continue;

        int srcx = f->x[*text - DA_ASCII_BEGIN];
        int srcy = f->y[*text - DA_ASCII_BEGIN];

        if (srcx == -1 || srcy == -1)
            continue;

        pixel_mover (
                srcx, srcy,
                x, y,
                min (f->w, w), min (f->h, h));

        w -= f->w;
        x += f->w;
    }

    return dockapp_rv_ok;
}












