/*
    wmget - A background download manager as a Window Maker dock app
    Copyright (c) 2001-2002 Aaron Trickey <aaron@amtrickey.net>

    Permission is hereby granted, free of charge, to any person
    obtaining a copy of this software and associated documentation files
    (the "Software"), to deal in the Software without restriction,
    including without limitation the rights to use, copy, modify, merge,
    publish, distribute, sublicense, and/or sell copies of the Software,
    and to permit persons to whom the Software is furnished to do so,
    subject to the following conditions:

    The above copyright notice and this permission notice shall be
    included in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
    NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

    ********************************************************************
    request.c - Client code for submitting download requests

    When the user runs wmget without the ``dock'' special parameter,
    main() calls request(), defined below.  This code parses the options
    and forms and submits a download job assuming everything's okay.
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <getopt.h>

#include "wmget.h"


/* Option constants, used in a case statement below.
 */
#define O(s,l,a,t,d)    OPT_##l = s,
enum {
#include "options.def"
};
#undef O


static void usage (void)
{
    printf (
WMGET_VERSION_BANNER "\n"
"Copyright (c) 2001-2002 Aaron Trickey; ABSOLUTELY NO WARRANTY\n"
"Usage:\n"
"   wmget dock\n"
"   wmget [job-options] <url>\n"
"   wmget cancel <jobnumber>\n"
"   wmget list\n"
" If `dock' is specified, launches the server which runs as a\n"
" dock app.  (Only one such server may be run per logged-in user.)\n"
" To request a download, use the second form, providing a URL and\n"
" whichever job options you want.  This returns you a download job\n"
" number; if you later want to cancel the job, you can use the third\n"
" form which accepts that job number and cancels it.  The fourth\n"
" form writes out the current download queue.\n"
"\n"
"Job options (second form only):\n");

#define required " X"
#define optional " [X]"
#define no       
#define O(s,l,a,t,d) \
    printf (" -%c|--%-12s " t "\n                   Default: " d "\n",\
            s, #l a);
#include "options.def"
#undef O
#undef no
#undef optional
#undef required
}


static const char *enquote_strdup (const char *string)
{
    int len = strlen (string);
    char *newstr;
    const char *src;
    char *dest;

    for (src = string; *src; src += strcspn (src, ")\\")) {
        ++len;
    }

    dest = newstr = malloc (len + 1);

    for (src = string; *src; ++src) {
        switch (*src) {
            case ')':
            case '\\':
                *dest++ = '\\';
        }

        *dest++ = *src;
    }

    return newstr;
}

int submit_job (Job *j)
{
    char line[MAXCMDLEN + 1];
    char *word_break;
    FILE *fp;

    const char *srcurl = enquote_strdup (j->srcurl);
    const char *display_name = enquote_strdup (j->display_name);
    const char *target = enquote_strdup (j->target);

    if (snprintf (line, MAXCMDLEN,
                  CMD_GET " "
                  ARG_GET_URL "(%s) "
                  ARG_GET_DISPLAY "(%s) "
                  ARG_GET_TARGET "(%s) "
                  ARG_GET_START_FROM "(%lu)"
                  ARG_GET_OVERWRITE_OK "(%d)\n",
                  srcurl, display_name, target,
                  (unsigned long)j->continue_download_from,
                  j->opt_overwrite) < 0) {

        error ("The URL, target, or *something* is too long for my\n"
               "command syntax!");

        free ((void *)srcurl);
        free ((void *)display_name);
        free ((void *)target);

        return 1;
    }

    free ((void *)srcurl);
    free ((void *)display_name);
    free ((void *)target);

    debug ("Command line is '%s'", line);

    if (!(fp = iq_client_connect ()))
        return 1;

    if (fputs (line, fp) == EOF) {
        error_sys ("Failed to send command to server (fputs)");
        fclose (fp);
        return 1;
    }

    if (!fgets (line, sizeof line - 1, fp)) {
        error ("Server did not respond to command!");
        fclose (fp);
        return 1;
    }

    /* Extract the first word and compare. */
    word_break = line + strcspn (line, " \t\r\n");

    while (*word_break && isspace (*word_break))
        *word_break++ = 0;

    if (strcasecmp (line, RESPONSE_JOB_ACCEPTED)) {
        error ("Server responded with error: %s", word_break);
        fclose (fp);
        return 1;
    }

    fclose (fp);

    /* Since we got a RESPONSE_JOB_ACCEPTED, present the user with the
     * job ID for future cancellation.
     */
    printf ("Request; job #%s\n", word_break);

    return 0;
}

int request (int argc, char *const argv[])
{
    Job j;
    int o;
    static const char shortopts[] = {
#define no
#define optional    , ':', ':'
#define required    , ':'
#define O(s,l,a,t,d) s a,
#include "options.def"
#undef O
#undef no
#undef optional
#undef required
        0
    };

    static const struct option longopts[] = {
#define O(s,l,a,t,d) { #l, a##_argument, 0, s },
#include "options.def"
#undef O
        {   0,              0,                  0,              0   }
    };

    j.continue_download_from = 0;
    j.opt_overwrite = 0;

    while ((o = getopt_long (argc, argv, shortopts, longopts, 0))
            != EOF) {
        switch (o) {
            default:
                error ("Unrecognized option.");
                usage ();
                return 1;

            case OPT_version:
                info (WMGET_VERSION_BANNER);
                return 0;

            case OPT_output:
                if (!optarg) {
                    error ("Missing argument to -o.");
                    usage ();
                    return 1;
                }
                strncpy (j.target, optarg, sizeof j.target - 1);
                j.target[sizeof j.target - 1] = 0;
                break;

            case OPT_display:
                if (!optarg) {
                    error ("Missing argument to -d.");
                    usage ();
                    return 1;
                }
                strncpy (j.display_name, optarg, sizeof j.display_name - 1);
                j.display_name[sizeof j.display_name - 1] = 0;
                break;

            case OPT_overwrite:
                j.opt_overwrite = 1;
                break;

            case OPT_continue:
                /* for now, just flag continue download: we'll set the
                 * actual bytecount when we stat the file in
                 * build_and_check_target()
                 */
                j.continue_download_from = 1;
                break;

            case OPT_silent:
                output_level = OL_SILENT;
                break;

            case OPT_verbose:
                output_level = OL_DEBUG;
                break;
        }
    }

    if (argc - optind > 1) {
        error ("Extra, unrecognized arguments.");
        usage ();
        return 1;

    } else if (argc - optind == 0) {
        error ("Missing URL.");
        usage ();
        return 1;
    }

    strncpy (j.srcurl, argv[optind], sizeof j.srcurl - 1);
    j.srcurl[sizeof j.srcurl - 1] = 0;

    if (build_and_check_target (&j))
        return 1;

    debug ("Submitting:");
    debug_dump_job (&j);

    if (submit_job (&j))
        return 1;

    return 0;
}

