/*
    wmget - A background download manager as a Window Maker dock app
    Copyright (c) 2001-2002 Aaron Trickey <aaron@amtrickey.net>

    Permission is hereby granted, free of charge, to any person
    obtaining a copy of this software and associated documentation files
    (the "Software"), to deal in the Software without restriction,
    including without limitation the rights to use, copy, modify, merge,
    publish, distribute, sublicense, and/or sell copies of the Software,
    and to permit persons to whom the Software is furnished to do so,
    subject to the following conditions:

    The above copyright notice and this permission notice shall be
    included in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
    NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

    ********************************************************************
    wmget.c - main() and a few common functions

    This file contains the main() for wmget; it simply checks for the
    presence of ``dock'' as the first argument and invokes server(),
    request(), cancel(), or list() as appropriate.  Also found here are
    some global functions, including home_directory() and all the error,
    warning, and message functions.
*/

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <pwd.h>
#include <string.h>
#include <stdarg.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "wmget.h"

enum OutputLevel output_level = OL_NORMAL;

void debug_dump_job (Job *job)
{
    if (output_level < OL_DEBUG)
        return;

    debug ("id = %lu\n"
           "status = %d\n"
           "progress=%d/%d\n"
           "stop_request=%d\n"
           "display_name=[%s]\n"
           "srcurl=[%s]\n"
           "target=[%s]\n"
           "continue_download_from=%d\n\n",
           job->job_id,
           job->status, job->progress, job->stop_request,
           job->prog_max, job->display_name, job->srcurl, job->target,
           job->continue_download_from);
}


const char *home_directory (void)
{
    static char *home_directory = 0;
    struct passwd *pwent;

    if (home_directory)
        return home_directory;

    home_directory = getenv ("HOME");

    if (!home_directory) {
        pwent = getpwuid (geteuid ());
        home_directory = strdup (pwent->pw_dir);
    }

    return home_directory;
}


int build_and_check_target (Job *j)
{
    struct stat st;
    const char *homedir = home_directory ();
    size_t homedirsz = strlen (homedir);
    const char *base_first;
    const char *base_last;
    size_t base_sz;

    switch (j->target[0]) {
        case '/':
            /* Target path exists, and is absolute.  Good. */
            break;

        case 0:
            /* No target path given at all.  Construct a basename
             * consisting of the last slash-delimited portion of the URL
             * (there has to be at least one), then fall through to the
             * default case where the user's home dir is prepended.
             */
            debug ("Constructing target from source [%s]", j->srcurl);

            base_last = j->srcurl + strlen (j->srcurl) - 1;
            while (*base_last == '/' && base_last > j->srcurl)
                --base_last;
            base_first = base_last;
            while (*base_first != '/' && base_first > j->srcurl)
                --base_first;
            base_sz = base_last - base_first;
            ++base_first;   /* get it past that initial slash */

            if (base_sz == 0) {
                /* Uh, oh... invalid srcurl anyway... give up. */
                error ("Invalid srcurl [%s], cannot build target...",
                       j->srcurl);
                return 1;
            }

            if (base_sz + 1 > sizeof j->target) {
                /* too big... truncate */
                base_sz = sizeof j->target - 1;
            }

            strncpy (j->target, base_first, base_sz);
            j->target[base_sz] = '\0';

            debug ("Got basename [%s]", j->target);

            /* FALLTHROUGH */

        default:
            /* Relative path.  Prepend user's home directory. */
            if (homedirsz + 2 + strlen (j->target) > MAXPATHLEN) {
                error ("Filename or home directory too long");
                return 1;
            }
            memmove (j->target + homedirsz + 1, j->target,
                     strlen (j->target) + 1);
            memcpy (j->target, homedir, homedirsz);
            j->target[homedirsz] = '/';
            break;
    }

    /* Now that we have a valid target, we can use that as a default for
     * the display name.  Select the basename.
     */
    if (j->display_name[0] == '\0') {
        char *target_base = strrchr (j->target, '/');
        if (target_base && target_base[1])
            ++target_base;
        else
            target_base = j->target;

        strncpy (j->display_name,
                target_base,
                sizeof j->display_name - 1);
        j->display_name[sizeof j->display_name - 1] = '\0';
    }

    /* test for writeability/existence */
    if (stat (j->target, &st)) {
        if (errno == ENOENT) {
            /* file does not exist.  Good. */
            return 0;
        }
        error_sys ("could not stat(`%s')", j->target);
        return 1;
    }

    if (!j->opt_overwrite && !j->continue_download_from) {
        error ("File '%s' exists, aborting.", j->target);
        info (
            "To overwrite an existing file, use --overwrite (-O).\n"
            "To continue an aborted download, use --continue (-C).\n");
        return 1;
    }

    /* for continuations, get the file length. */
    if (j->continue_download_from) {
        if (!S_ISREG (st.st_mode)) {
            error (
                "I can't continue (-C) a download unless the existing"
                "file is a regular, seekable, file.\n");
            return 1;
        }
        j->continue_download_from = st.st_size;
    }

    /* finally, check permissions */
    if ((st.st_mode & S_IWOTH)
        || ((st.st_mode & S_IWGRP) && st.st_gid == getegid ())
        || ((st.st_mode & S_IWUSR) && st.st_uid == geteuid ()))
        return 0;

    error ("File '%s' exists and is not writable.\n", j->target);

    return 1;
}


extern void error (const char *fmt, ...)
{
    va_list ap;
    va_start (ap, fmt);
    vfprintf (stderr, fmt, ap);
    va_end (ap);
    fputs ("\n", stderr);
}

extern void error_sys (const char *fmt, ...)
{
    va_list ap;
    va_start (ap, fmt);
    vfprintf (stderr, fmt, ap);
    va_end (ap);
    perror (": ");
}

extern void info (const char *fmt, ...)
{
    va_list ap;

    if (output_level < OL_NORMAL)
        return;

    va_start (ap, fmt);
    vfprintf (stderr, fmt, ap);
    va_end (ap);
    fputs ("\n", stderr);
}

extern void debug (const char *fmt, ...)
{
    va_list ap;

    if (output_level < OL_DEBUG)
        return;

    va_start (ap, fmt);
    vfprintf (stderr, fmt, ap);
    va_end (ap);
    fputs ("\n", stderr);
}


int main (int argc, char *argv[])
{
    if (argc > 1 && strcmp (argv[1], "dock") == 0) {
        return server (argc, argv);
    } else if (argc > 1 && strcmp (argv[1], "cancel") == 0) {
        return cancel (argc, argv);
    } else if (argc > 1 && strcmp (argv[1], "list") == 0) {
        return list (argc, argv);
    } else
        return request (argc, argv);
}








